@@ -1,3 +1,4 @@
+import csv
 import datetime
 import logging
 from time import sleep
@@ -5,6 +6,7 @@ from time import sleep
 import requests
 
 from ravenpackapi.exceptions import api_method, APIException, DataFileTimeout
+from ravenpackapi.util import to_curl, parse_csv_line
 
 logger = logging.getLogger(__name__)
 
@@ -36,6 +38,10 @@ class Job(object):
     def is_processing(self):
         return self.status in {'enqueued', 'processing'}
 
+    @property
+    def is_undefined(self):
+        return self.status in {'unknown'}
+
     def __str__(self):
         return "Job {status}: {token}".format(status=self.status,
                                               token=self.token)
@@ -57,9 +63,14 @@ class Job(object):
         max_end_date = datetime.datetime.utcnow() + datetime.timedelta(
             seconds=timeout_seconds
         ) if timeout_seconds else None
+
+        if self.is_undefined:
+            self.get_status()
+
         while True:
             if self.is_ready:
                 break
+            sleep(self._FILE_AVAILABILIY_SECONDS_DELAY)
             try:
                 self.get_status()
             except APIException:  # keep waiting if API raises exceptions
@@ -70,9 +81,8 @@ class Job(object):
                     "Timeout: job wasn't complete after %d seconds" % timeout_seconds
                 )
             if not printed_once:
-                logger.info("Waiting for the job to be ready...")
+                logger.info("Waiting for the job %s to be ready..." % self.token)
                 printed_once = True
-            sleep(self._FILE_AVAILABILIY_SECONDS_DELAY)
 
     @api_method
     def save_to_file(self, filename):
@@ -84,11 +94,39 @@ class Job(object):
 
             # this is a different request than the normal API
             # streaming the file in chunks
-            r = requests.get(job.url,
-                             headers=dict(API_KEY=api.api_key),
-                             stream=True,
-                             )
-
-            for chunk in r.iter_content(chunk_size=self._CHUNK_SIZE):
+            response = requests.get(job.url,
+                                    headers=api.headers,
+                                    stream=True,
+                                    )
+            if response.status_code != 200:
+                logger.error("Error calling the API, we tried: %s" % to_curl(response.request))
+                raise APIException(
+                    'Got an error {status}: body was \'{error_message}\''.format(
+                        status=response.status_code, error_message=response.text
+                    ), response=response)
+            for chunk in response.iter_content(chunk_size=self._CHUNK_SIZE):
                 if chunk:
                     output.write(chunk)
+
+    @api_method
+    def iterate_results(self):
+        api = self.api
+        job = self  # just to be clear
+        job.wait_for_completion()
+
+        with requests.Session() as s:
+            r = s.get(job.url,
+                      headers=api.headers,
+                      stream=True,
+                      )
+            iterator = r.iter_lines(chunk_size=self._CHUNK_SIZE)
+            headers = next(iterator)  # discard the headers
+
+            for line in iterator:
+                fields = parse_csv_line(line)
+                yield fields
+
+    def __iter__(self):
+        # this will be yield from in Py3
+        for record in self.iterate_results():
+            yield record
